'----------------------
 '  ..::WarpSounds::..
 '----------------------
 'Author: Nikolas Gbel
 '
 '----------------------

 SuperStrict 

 Module comnik.warpsound

 ModuleInfo "Framework: This Module provides extended sound effects for BlitzMax."
 ModuleInfo "Author: Nikolas 'ComNik' Gbel"
 ModuleInfo "Version: 1.1"
 ModuleInfo "License: Feel free to use the code as you like, but it would be nice to mention my name... ;)"
 
 Import BRL.LinkedList
 Import BRL.System 
 Import BRL.Audio
 Import BRL.Math

 Global S_emitter:TList = CreateList()
 Rem
   bbdoc: Sound Emitter
   about: A sound emitter contains a channel, which he plays, 
          if he gets activated by a trigger in his trigger list. 
          If the trigger gets closer, the sound gets louder.
 End Rem  
 Type WP_SoundEmitter
  Field trigger:TList
  Field channel:TChannel  
  
  Field x:Float ' Position in the 2D World.
  Field y:Float
  Field aDistance:Float 'Activation distance. 
  Field maxvol:Float
  Field loop:Byte 

  Rem
    bbdoc: Remove a Trigger.
  End Rem 
  Method WP_RemoveSoundTrigger(trigger:WP_SoundTrigger)
   ListRemove(Self.trigger, trigger)
  End Method  

  Rem
    bbdoc: Kill the Emitter.
    about: Stops the Channel, and removes the Emitter.
  End Rem 
  Method kill()
   Local trigg:WP_SoundTrigger
   For trigg = EachIn trigger
    ListRemove(Self.trigger, trigg)
   Next 

   StopChannel(channel)
   ListRemove(S_emitter, Self)
  End Method    
 End Type 

  Rem
    bbdoc: Create a sound emitter.
    returns: A sound emitter object.
  End Rem 
  Function WP_CreateSoundEmitter:WP_SoundEmitter(channel:TChannel, x:Float, y:Float, activation_distance:Float=800, MAXVOL:Float=1, LOOP:Byte=False)
   Local se:WP_SoundEmitter
   se = New WP_SoundEmitter
    se.trigger = CreateList()
    se.channel = channel
    
    se.x = x
    se.y = y
    se.aDistance = activation_distance
    se.loop = LOOP
    If MAXVOL <= 1
     se.maxvol = MAXVOL
    Else
     Notify("Maxvol can only bee between 0 and 1!"); End 
    End If 
   ListAddLast(S_emitter, se)
   Return se
  End Function

  Rem
    bbdoc: Add a new Trigger
    returns: A Sound Trigger object.
    about: Creates a new trigger and returns it's handle.
  End Rem   
  Function WP_AddSoundTrigger:WP_SoundTrigger(x:Float, y:Float, parent:WP_SoundEmitter) 
   Local st:WP_SoundTrigger
   st = New WP_SoundTrigger
    st.x = x
    st.y = y
    st.InRange = False 
    st.parent = parent
   ListAddLast(parent.trigger, st)
   Return st
  End Function  

 Rem
   bbdoc: Sound Trigger
   about: Activates his parent Emitter, if the distance between them is
          less than the activation distance of the emitter.
 End Rem 
 Type WP_SoundTrigger
  Field parent:WP_SoundEmitter
  
  Field x:Float
  Field y:Float  

  Field InRange:Byte 
 End Type 
  
 Rem
   bbdoc: Update triggers and emitters.
 End Rem 
 Function WP_UpdateSounds()
  Local emitter:WP_SoundEmitter
  Local trigger:WP_SoundTrigger
  For emitter = EachIn S_emitter
   For trigger = EachIn emitter.trigger
    trigger.InRange = InRange(trigger.x, trigger.y, trigger.parent.x, trigger.parent.y, trigger.parent.aDistance)
    If trigger.InRange = True 
     If trigger.parent.loop = True Then ResumeChannel(trigger.parent.channel)
     Local dist:Float = Sqr((trigger.x-trigger.parent.x)*(trigger.x-trigger.parent.x)+(trigger.y-trigger.parent.y)*(trigger.y-trigger.parent.y))
     Local vol:Float = (trigger.parent.aDistance-dist)*0.001
     If IsLeft(trigger.x, trigger.y, trigger.parent.x, trigger.parent.y)
      SetChannelPan(trigger.parent.channel, -vol)
     Else 
      SetChannelPan(trigger.parent.channel, vol)
     End If 
     If vol <= trigger.parent.maxvol Then SetChannelVolume(trigger.parent.channel, vol)
    Else
     If trigger.parent.loop = True Then PauseChannel(trigger.parent.channel)
     SetChannelVolume(trigger.parent.channel, 0)
    End If 
   Next  
  Next  
 End Function   

 Rem
   bbdoc: Check if two distances "overlap"
 End Rem 
 Function InRange:Byte(x1:Float, y1:Float, x2:Float, y2:Float, dist:Float)
  If (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1) < dist^2 Then Return True ; Return False 
 End Function 

 Rem
   bbdoc: Check if position 1 is left in relation to position 2.
 End Rem 
 Function IsLeft:Byte(x1:Float, y1:Float, x2:Float, y2:Float)
  If x1 < x2 Return True; Return False
 End Function 

 Rem
   bbdoc: Check if position 1 is right in relation to position 2.
 End Rem 
 Function IsRight:Byte(x1:Float, y1:Float, x2:Float, y2:Float)
  If x1 > x2 Then Return True; Return False 
 End Function 
